from .config import Config
from .accounts import Accounts


class Shared():

    def __init__(self, config: Config, accounts: Accounts, environment):

        fqdn_as_list = self.get_fqdn_as_list(config, environment)

        self.fqdn = ".".join(fqdn_as_list)
        self.fqdn_reverse = ".".join(reverse(fqdn_as_list))
        self.fqdn_reverse_dash = "-".join(reverse(fqdn_as_list))
        self.environment_domain_name = ".".join(fqdn_as_list[1:])
        self.environment_domain_name_reverse = ".".join(
            reverse(fqdn_as_list[1:]))
        self.environment_domain_name_reverse_dash = "-".join(
            reverse(fqdn_as_list[1:]))
        self.fqdn_no_env = ".".join(fqdn_as_list[2:])
        self.fqdn_no_env_reverse = ".".join(reverse(fqdn_as_list[2:]))
        self.fqdn_no_env_reverse_dash = "-".join(reverse(fqdn_as_list[2:]))

        self.web_acl_name = "IPWhiteListWebACL"
        self.aws_role_arn = f"arn:aws:iam::{accounts.terraform_state_account_id}:role/{config.app_name}-terraform-pipleine-CodeBuildRole"

        self.aws_profile = f"{accounts.aws_account_id}_AWSPowerUserAccess"
        self.build_account_profile = f"{accounts.build_account_id}_AWSPowerUserAccess"
        self.dns_account_profile = f"{accounts.dns_account_id}_NetworkAdministrator"

        self.artifacts_bucket_name = f"{config.tldn}.{environment}.artifacts"
        self.destination_builds_bucket_name = f"{config.tldn}.{environment}.builds"
        self.source_build_bucket_name = f"{config.tldn}.{accounts.build_account_name}.builds"
        self.cloudtrails_logs_bucket_name = f"{config.tldn}.{environment}.cloudtrails.logs"
        self.terraform_bucket_name = f"{config.tldn}.{accounts.terraform_state_account_name}.terraform"
        self.aws_sns_topic_env_build_notification_name = f"{self.environment_domain_name_reverse_dash}-build-notifications"
        self.aws_sns_topic_build_notification_name = f"{self.fqdn_no_env_reverse_dash}-build-notifications"
        self.terraform_dynamodb_table = f"{config.tldn}.{accounts.terraform_state_account_name}.terraform.lock"
        self.destination_builds_bucket_name = f"{config.tldn}.{environment}.builds"
        self.common_tags = {
            "project_name": config.project_name,
            "application_name": config.app_name,
            "domain": self.fqdn,
            "workspace": environment,
            "environment_name": environment
        }

    def get_fqdn_as_list(self, config, environment):
        fqdn_as_list = None
        tldn_as_list = config.tldn.split(".")[::-1]
        if environment == "prod":
            fqdn_as_list = [config.app_name] + tldn_as_list
        else:
            fqdn_as_list = [config.app_name, environment] + tldn_as_list

        fqdn_as_list = compact(fqdn_as_list)
        return fqdn_as_list


def reverse(str_list):
    return list(reversed(str_list))


def compact(str_list):
    return list(filter(None, str_list))



import boto3
import json


class Accounts():

    def __init__(self, environment, accounts_profile):
        super().__init__()
        self._client = None

        self._accounts_profile = accounts_profile

        locals = self._get_locals_from_account_tags(environment)
        self.aws_account_id = locals.get("aws_account_id")
        self.dns_account_id = locals.get("dns_account_id")
        self.build_account_id = locals.get("build_account_id")
        self.build_account_name = locals.get("build_account_name")
        self.terraform_state_account_name = locals.get(
            "terraform_state_account_name")
        self.terraform_state_account_id = locals.get(
            "terraform_state_account_id")
        self.build_children_account_ids = locals.get(
            "build_children_account_ids")
        self.terraform_state_children_account_ids = locals.get(
            "terraform_state_children_account_ids")
        self.dns_children_account_ids = locals.get(
            "dns_children_account_ids")
        self.locals = locals

    def _get_locals_from_account_tags(self, environment):
        accounts, account_ids = self.get_account_ids()

        locals = {}

        for account in accounts:
            account_name = account['Name']
            account_id = account['Id']

            if account_name == environment:
                locals["aws_account_id"] = account_id

            tags = self.get_client().list_tags_for_resource(ResourceId=account_id)
            for tag in tags['Tags']:
                key = tag['Key'].replace('-', '_')
                value = tag['Value']
                children_key = key + '_children_account_ids'
                local_account_id_key = key + '_account_id'
                local_account_name_key = key + "_account_name"

                if value == environment:
                    if children_key not in locals:
                        locals[children_key] = []
                    locals[children_key].append(account_id)

                if account_name == environment:
                    locals[local_account_id_key] = account_ids[value]
                    locals[local_account_name_key] = value

        return locals

    def get_account_ids(self):
        client = self.get_client()
        accounts = client.list_accounts()['Accounts']
        account_ids = {}
        for account in accounts:
            account_ids[account['Name']] = account['Id']

        return accounts, account_ids

    def get_client(self):
        if not self._client:
            session = boto3.Session(profile_name=self._accounts_profile)
            self._client = session.client('organizations')
        return self._client



from cdktf_cdktf_provider_aws import AwsProvider, AwsProviderAssumeRole

from .config import Config


class ProviderFactory():
    def __init__(self, stack, config: Config, aws_role_arn, aws_profile):
        self._stack = stack
        self._config = config

        self._profile = None
        self._assume_role = None
        if config.use_role_arn:
            self._assume_role = AwsProviderAssumeRole(
                stack, role_arn=aws_role_arn)
        else:
            self._profile = aws_profile

    def build(self, aws_region, id="aws", alias=None, profile=None):
        if not profile:
            profile = self._profile

        return AwsProvider(
            self._stack, id=id, region=aws_region, profile=profile, assume_role=self._assume_role, alias=alias
        )



import json
from mock import patch

from terraform.accounts import Accounts


def read_json_file(filename):
    return json.loads(open(filename, "r").read())


@patch('terraform.accounts.boto3')
def test_service_accounts_info(mock_boto3):
    # Arrange
    session = mock_boto3.Session.return_value
    client = session.client.return_value
    client.list_accounts.return_value = read_json_file(
        "terraform/tests/list_accounts_response.json")

    client.list_tags_for_resource.side_effect = [
        json.loads(
            '{ "Tags":[{"Key": "dns", "Value": "dns"}, {"Key": "build", "Value": "build"}, {"Key": "terraform_state", "Value": "build"}]}'),
        json.loads('{ "Tags":[]}'),
        json.loads('{ "Tags":[]}')
    ]

    # Act
    accounts = Accounts("endtoend", "453254632971_ListAccountsAccess")

    # Assert
    assert accounts != None
    assert accounts.aws_account_id == "123456789012"
    assert accounts.build_account_id == "210987654321"
    assert accounts.build_account_name == "build"
    assert accounts.dns_account_id == "567890123456"
    assert accounts.terraform_state_account_id == "210987654321"
    assert accounts.terraform_state_account_name == "build"
    assert accounts.build_children_account_ids == None
    assert accounts.terraform_state_children_account_ids == None
    assert accounts.dns_children_account_ids == None


@patch('terraform.accounts.boto3')
def test_build_accounts_info(mock_boto3):
    # Arrange
    session = mock_boto3.Session.return_value
    client = session.client.return_value
    client.list_accounts.return_value = read_json_file(
        "terraform/tests/list_accounts_response.json")

    client.list_tags_for_resource.side_effect = [
        json.loads(
            '{ "Tags":[{"Key": "dns", "Value": "dns"}, {"Key": "build", "Value": "build"}, {"Key": "terraform_state", "Value": "build"}]}'),
        json.loads(
            '{ "Tags":[{"Key": "build", "Value": "build"}, {"Key": "terraform_state", "Value": "build"}]}'),
        json.loads('{ "Tags":[{"Key": "build", "Value": "build"}]}')
    ]

    # Act
    accounts = Accounts("build", "453254632971_ListAccountsAccess")

    # Assert
    assert accounts != None
    assert accounts.aws_account_id == "210987654321"
    assert accounts.build_account_id == "210987654321"
    assert accounts.build_account_name == "build"
    assert accounts.dns_account_id == None
    assert accounts.terraform_state_account_id == "210987654321"
    assert accounts.terraform_state_account_name == "build"
    assert accounts.build_children_account_ids == [
        "123456789012", "210987654321", "567890123456"]
    assert accounts.terraform_state_children_account_ids == [
        "123456789012", "210987654321"]
    assert accounts.dns_children_account_ids == None


@patch('terraform.accounts.boto3')
def test_dns_accounts_info(mock_boto3):
    # Arrange
    session = mock_boto3.Session.return_value
    client = session.client.return_value
    client.list_accounts.return_value = read_json_file(
        "terraform/tests/list_accounts_response.json")

    client.list_tags_for_resource.side_effect = [
        json.loads(
            '{ "Tags":[{"Key": "dns", "Value": "dns"}, {"Key": "build", "Value": "build"}, {"Key": "terraform_state", "Value": "build"}]}'),
        json.loads(
            '{ "Tags":[{"Key": "build", "Value": "build"}, {"Key": "terraform_state", "Value": "build"}]}'),
        json.loads(
            '{ "Tags":[{"Key": "build", "Value": "build"}, {"Key": "terraform_state", "Value": "build"}]}')
    ]

    # Act
    accounts = Accounts("dns", "453254632971_ListAccountsAccess")

    # Assert
    assert accounts != None
    assert accounts.aws_account_id == "567890123456"
    assert accounts.build_account_id == "210987654321"
    assert accounts.build_account_name == "build"
    assert accounts.dns_account_id == None
    assert accounts.terraform_state_account_id == "210987654321"
    assert accounts.terraform_state_account_name == "build"
    assert accounts.build_children_account_ids == None
    assert accounts.terraform_state_children_account_ids == None
    assert accounts.dns_children_account_ids == ["123456789012"]



from constructs import Node, Construct

import os


class Config():

    def __init__(self, app: Construct, ns: str):
        self._scope = app
        self._ns = ns

        self.accounts_profile = self._get_accounts_profile()
        self.tldn = self._get_top_level_domain_name()
        self.use_role_arn = self._get_use_terraform_state_role_arn()
        self.app_name = self._get_app_name()
        self.aws_region = self._get_aws_region()
        self.project_name = self._project_name()

    def _get_use_terraform_state_role_arn(self):
        return self._get_config("use_terraform_state_role_arn", "CDKTF_USE_TERRAFORM_STATE_ROLE_ARN", False)

    def _get_top_level_domain_name(self):
        return self._get_config("top_level_domain_name", "CDKTF_TOP_LEVEL_DOMAIN_NAME")

    def _get_accounts_profile(self):
        return self._get_config("accounts_profile", "CDKTF_ACCOUNT_PROFILE")

    def _get_app_name(self):
        return self._get_config("app_name", "CDKTF_APP_NAME")

    def _get_aws_region(self):
        return self._get_config("aws_region", "CDKTF_AWS_REGION")

    def _project_name(self):
        return self._get_config("project_name", "CDKTF_PROJECT_NAME")

    def _get_config(self, context_name, environment_variable_name, default=None):
        node = Node.of(self._scope)
        config = os.environ.get(environment_variable_name)
        config = config or node.try_get_context(context_name) or default
        return config



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['murraytait_cdktf', 'murraytait_cdktf.tests']

package_data = \
{'': ['*']}

install_requires = \
['boto3>=1.20.31,<2.0.0',
 'cdktf-cdktf-provider-aws>=4.0.2,<5.0.0',
 'cdktf>=0.8.6,<0.9.0',
 'constructs>=10.0.29,<11.0.0',
 'pytest-mock>=3.6.1,<4.0.0']

setup_kwargs = {
    'name': 'murraytait-cdktf',
    'version': '0.1.2',
    'description': '',
    'long_description': None,
    'author': 'Murray Tait',
    'author_email': '6074789+deathtumble@users.noreply.github.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
